#include "LineTool.h"

#define _USE_MATH_DEFINES
#include <math.h>

#include "Polygon.h"
#include "PolyDraw.h"

LineTool::~LineTool()
{
	clear();
}

LineTool::LineTool(PolyDraw *pd) : Tool(pd),line(NULL),normal(NULL),intersection(NULL)
{
	draw->polygons.splice(draw->polygons.end(),draw->selected_polygons);
}

bool LineTool::on_mouse_down(GdkEventButton *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	// start point for line
	if(event->button == 1)
	{
		if(line == NULL)
		{
			line = new Line(Vertex(x,-y),Vertex(x,-y),Line::Segment);
		}
	}
	return true;
}

bool LineTool::on_mouse_up(GdkEventButton *event)
{
	// delete line
	if(event->button == 1)
	{
		clear();
		line = NULL;
		normal = NULL;
		intersection = NULL;
		return true;
	}

	return false;
}

bool LineTool::on_mouse_move(GdkEventMotion *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	if(!line)	return false;

	line->B(Vertex(x,-y));
	// intersections?
	std::list<Intersection> intersections;
		
	for(std::list<Polygon *>::const_iterator it = draw->polygons.begin(); it != draw->polygons.end(); ++it)
	{
		(*it)->intersects(*line, intersections, true);
	}
	delete intersection;
	delete normal;
	intersection = NULL;
	normal = NULL;
	// get closest one
	Intersection *closestIntersection = NULL;
	for(std::list<Intersection>::iterator it = intersections.begin(); it != intersections.end(); ++it)
	{
		if(closestIntersection == NULL || closestIntersection->otherParam > it->otherParam)
			closestIntersection = &(*it);
	}
	if(closestIntersection != NULL) {
		Intersection in = *closestIntersection;
		if (in.line != NULL) {
			double one, two;
			line->contains(in.line->A(),one);
			line->contains(in.line->B(),two);
			if (one < two)
				intersection = new Vertex(in.line->A());
			else
				intersection = new Vertex(in.line->B());
		} else {
			intersection = new Vertex(*in.point);
		}
		Line li(Vertex(0,0),in.thisNormal);
		normal = new Vertex(in.thisNormal * 50 / li.getLength());
	}
	return true;
}

bool LineTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	if(!line) return false;

	double size = draw->size;

	cr->set_line_width(2.0/size);
	if(line != NULL) 
	{
		cr->set_source_rgba(0.3, 0.3, 0.6, 0.8);
		cr->move_to(line->A().x()/size, -line->A().y()/size);
		cr->line_to(line->B().x()/size, -line->B().y()/size);
		cr->stroke();
	}
	if(normal != NULL && intersection != NULL)
	{
		cr->set_source_rgba(0.6, 0.3, 0.3, 0.8);
		cr->move_to(intersection->x()/size, -intersection->y()/size);
		cr->line_to((intersection->x()+normal->x())/size, -(intersection->y()+normal->y())/size);
		cr->stroke();
	}
	if(intersection != NULL)
	{
		cr->set_source_rgba(0.6, 0.3, 0.3, 0.8);
		cr->arc (intersection->x()/size, -intersection->y()/size, 5.0/size, 0, 2*M_PI);
		cr->stroke();
	}
	return true;
}

void LineTool::clear()
{
	delete line;
	delete normal;
	delete intersection;
}