#define _USE_MATH_DEFINES
#include <cmath>
#include <cairomm/context.h>
#include <gdk/gdkkeysyms.h>

#include "PolyDraw.h"

PolyDraw::PolyDraw()
{
	size = 600.0;

	tool = NULL;

  set_flags(Gtk::CAN_FOCUS);

  add_events( Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::POINTER_MOTION_MASK);
  signal_button_press_event().connect( sigc::mem_fun( *this, &PolyDraw::on_mouse_down ) );
	signal_button_release_event().connect( sigc::mem_fun( *this, &PolyDraw::on_mouse_up ) );
  signal_motion_notify_event().connect( sigc::mem_fun( *this, &PolyDraw::on_move ) );

  #ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
  //Connect the signal handler if it isn't already a virtual method override:
  signal_expose_event().connect(sigc::mem_fun(*this, &PolyDraw::on_expose_event), false);
  #endif //GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
}

PolyDraw::~PolyDraw()
{
}

void PolyDraw::redraw()
{
	// redraw
  Gtk::Allocation allocation = get_allocation();
  const int width = allocation.get_width();
  const int height = allocation.get_height();
	Glib::RefPtr<Gdk::Window> win = get_window();
	if (win)
	{
		Gdk::Rectangle r(0, 0, width, height);
		win->invalidate_rect(r, false);
	}
}

bool PolyDraw::on_mouse_up( GdkEventButton *event )
{
	if(!event) return true;

	// tools...
	if(tool)
		if(tool->on_mouse_up(event))
			redraw();

	return true;
}

bool PolyDraw::on_mouse_down( GdkEventButton *event )
{
	if(!event) return true;

	// tools...
	if(tool)
		if(tool->on_mouse_down(event))
			redraw();

  return true;
}


bool PolyDraw::on_move( GdkEventMotion *event )
{
	if(!event) return true;
	
	// tool
	if(tool)
		if(tool->on_mouse_move(event))
			redraw();

	return true;
}

bool PolyDraw::on_expose_event(GdkEventExpose* event)
{

  // This is where we draw on the window
  Glib::RefPtr<Gdk::Window> window = get_window();
  if(window)
  {
    Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();

    if(event)
    {
        // clip to the area indicated by the expose event so that we only
        // redraw the portion of the window that needs to be redrawn
        cr->rectangle(event->area.x, event->area.y,
                event->area.width, event->area.height);
        cr->clip();
    }

    // scale to unit square and translate (0, 0) to be (0.5, 0.5), i.e.
    // the center of the window
    cr->scale(size, size);

		// draw background
		cr->set_source_rgba(1.0, 1.0, 0.8, 1.0);
		cr->paint();

		// draw all polygons
		for(std::list<Polygon *>::const_iterator pit = polygons.begin(); pit != polygons.end(); ++pit)
		{
			const std::vector<Vertex>& vertices = (*pit)->vertices();

			cr->set_line_width(1.0/size);
			cr->move_to(vertices.front().x()/size, -(vertices.front().y())/size);

			for(std::vector<Vertex>::const_iterator vit = vertices.begin(); vit != vertices.end(); ++vit)
			{
				cr->line_to(vit->x()/size, -vit->y()/size);
			}
			cr->close_path();
			cr->set_source_rgba(0.0, 0.4, 0.0, 0.8);
			cr->stroke_preserve();
			cr->set_source_rgba(0.2, 0.8, 0.2, 0.8);
			cr->fill();
		}

		for(std::list<Polygon *>::const_iterator pit = selected_polygons.begin(); pit != selected_polygons.end(); ++pit)
		{
			const std::vector<Vertex>& vertices = (*pit)->vertices();

			cr->set_line_width(1.0/size);
			cr->move_to(vertices.front().x()/size, -(vertices.front().y())/size);

			for(std::vector<Vertex>::const_iterator vit = vertices.begin(); vit != vertices.end(); ++vit)
			{
				cr->line_to(vit->x()/size, -vit->y()/size);
			}
			cr->close_path();
			cr->set_source_rgba(0.2, 0.6, 0.2, 0.8);
			cr->stroke_preserve();
			cr->set_source_rgba(0.3, 1.0, 0.3, 0.8);
			cr->fill();
		}

		if(tool)
			tool->redraw(cr);
  }
  return true;
}

void PolyDraw::setMode(Tool *t)
{
	// delete previous tool
	if(tool) delete tool;
	tool = t;
	redraw();
}