/*
 * 2D Polygon Landscape
 *
 * Copyright (c) 2011  Tobias Zwick
 */

#ifndef _VERTEX_H_
#define _VERTEX_H_

/* Vertex

   Represents a vertex, vector or point in 2D space.
  
 */
class Vertex
{
public:	
	Vertex(const Vertex& v);
	Vertex(int xpos, int ypos);
	Vertex();

	bool operator==(const Vertex& v1) const;
	bool operator!=(const Vertex& v1) const;

	Vertex& operator+=(const Vertex& v2);
	Vertex& operator-=(const Vertex& v2);
	Vertex operator+(const Vertex& v2) const;
	Vertex operator-(const Vertex& v2) const;

	Vertex& operator*=(const int v);
	Vertex& operator/=(const int v);
	Vertex operator*(const int v) const;
	Vertex operator/(const int v) const;

	Vertex& operator*=(const double v);
	Vertex& operator/=(const double v);
	Vertex operator*(const double v) const;
	Vertex operator/(const double v) const;

	Vertex operator+() const;
	Vertex operator-() const;

	/* scalar product */
	int operator*(const Vertex& v) const;

	/* get x-coordinate */
	const int& x() const { return _x; }
	/* set x-coordinate */
  void x(const int& val) { _x = val; }
	/* get y-coordinate */
	const int& y() const { return _y; }
	/* set y-coordinate */
  void y(const int& val) { _y = val; }

private:
	int _x,_y;
};

#endif

