/*
 * 2D Polygon Landscape
 *
 * Copyright (c) 2011  Tobias Zwick
 */

#ifndef _RINGVEC_H_
#define _RINGVEC_H_

#include <vector>
#include <list>

// lil helpers: (all copypasta! :-( )

/* For List */

// decrease iterator and hop over beginning to the end
template <typename T> typename std::list<T>::iterator decreaseIt(const typename std::list<T>::iterator &it, std::list<T> &alist, int shiftby = 1)
{
	typename std::list<T>::iterator newit = it;
	while(shiftby-- > 0)
	{
		if(newit == alist.begin()) newit = alist.end();
		--newit;
	}
	return newit;
}

// increase iterator and hop over end to the beginning
template <typename T> typename std::list<T>::iterator increaseIt(const typename std::list<T>::iterator &it, std::list<T> &alist, int shiftby = 1)
{
	typename std::list<T>::iterator newit = it;
	while(shiftby-- > 0)
	{
		++newit;
		if(newit == alist.end()) newit = alist.begin();
	}
	return newit;
}
// same same with const_...

// decrease iterator and hop over beginning to the end
template <typename T> typename std::list<T>::const_iterator decreaseIt(const typename std::list<T>::const_iterator &it, const std::list<T> &alist, int shiftby = 1)
{
	typename std::list<T>::const_iterator newit = it;
	while(shiftby-- > 0)
	{
		if(newit == alist.begin()) newit = alist.end();
		--newit;
	}
	return newit;
}

// increase iterator and hop over end to the beginning
template <typename T> typename std::list<T>::const_iterator increaseIt(const typename std::list<T>::const_iterator &it, const std::list<T> &alist, int shiftby = 1)
{
	typename std::list<T>::const_iterator newit = it;
	while(shiftby-- > 0)
	{
		++newit;
		if(newit == alist.end()) newit = alist.begin();
	}
	return newit;
}

/* For Vector */

// decrease iterator and hop over beginning to the end
template <typename T> typename std::vector<T>::iterator decreaseIt(const typename std::vector<T>::iterator &it, std::vector<T> &alist, int shiftby = 1)
{
	typename std::vector<T>::iterator newit = it;
	while(shiftby-- > 0)
	{
		if(newit == alist.begin()) newit = alist.end();
		--newit;
	}
	return newit;
}

// increase iterator and hop over end to the beginning
template <typename T> typename std::vector<T>::iterator increaseIt(const typename std::vector<T>::iterator &it, std::vector<T> &alist, int shiftby = 1)
{
	typename std::vector<T>::iterator newit = it;
	while(shiftby-- > 0)
	{
		++newit;
		if(newit == alist.end()) newit = alist.begin();
	}
	return newit;
}
// same same with const_...

// decrease iterator and hop over beginning to the end
template <typename T> typename std::vector<T>::const_iterator decreaseIt(const typename std::vector<T>::const_iterator &it, const std::vector<T> &alist, int shiftby = 1)
{
	typename std::vector<T>::const_iterator newit = it;
	while(shiftby-- > 0)
	{
		if(newit == alist.begin()) newit = alist.end();
		--newit;
	}
	return newit;
}

// increase iterator and hop over end to the beginning
template <typename T> typename std::vector<T>::const_iterator increaseIt(const typename std::vector<T>::const_iterator &it, const std::vector<T> &alist, int shiftby = 1)
{
	typename std::vector<T>::const_iterator newit = it;
	while(shiftby-- > 0)
	{
		++newit;
		if(newit == alist.end()) newit = alist.begin();
	}
	return newit;
}

#endif